@@ -17,6 +17,7 @@ from kabaret.subprocess_manager.flow import RunAction
 from .maputils import SimpleCreateAction, ClearMapAction
 
 from .site import SyncMap, Request
+from .runners import LaunchSessionWorker
 
 class RevisionsChoiceValue(flow.values.ChoiceValue):
 
@@ -81,6 +82,61 @@ class MakeCurrentRevisionAction(flow.Action):
         file.touch()
 
 
+class OpenRevision(RunAction):
+    _revision = flow.Parent()
+
+    def needs_dialog(self):
+        return False
+    
+    def extra_argv(self):
+        return [self._revision.get_path()]
+    
+    def runner_name_and_tags(self):
+        default_runners = self.root().project().admin.default_applications
+        runner = default_runners[self._revision._file.format.get()]
+        
+        return runner.runner_name.get(), []
+
+
+class ComputeRevisionHash(LaunchSessionWorker):
+    _revision = flow.Parent()
+
+    def allow_context(self, context):
+        return False
+
+    def launcher_oid(self):
+        return self._revision.oid()
+
+    def launcher_exec_func_name(self):
+        return "update_hash"
+
+
+class CheckRevisionHash(flow.Action):
+    _revision = flow.Parent()
+
+    def get_buttons(self):
+        self.message.revert_to_default()
+        return ["Check", "Close"]
+    
+    def run(self, button):
+        if button == "Close":
+            return
+
+        if self._revision.hash_is_valid():
+            color = "029600"
+            msg = "Hash is valid !"
+        else:
+            color = "D5000D"
+            msg = "Invalid hash"
+
+        self.message.set((
+            f"<h3><font color=#{color}>"
+            f"{msg}</font></h3>"
+        ))
+
+        return self.get_result(close=False)
+
+
 class Revision(flow.Object):
 
     _revisions = flow.Parent()
@@ -89,16 +145,19 @@ class Revision(flow.Object):
     user = flow.Param().ui(editable=False)
     date = flow.IntParam().ui(editable=False, editor="datetime")
     comment = flow.Param("").ui(editable=False)
-    path = flow.Computed()
+    path = flow.Computed(cached=True)
     file_name = flow.Computed()
     hash = flow.Param("").ui(editable=False)
 
     site = flow.Param()
     sync = flow.Child(SyncMap).ui(expanded=True)
 
+    open = flow.Child(OpenRevision)
     make_current = flow.Child(MakeCurrentRevisionAction)
     create_working_copy = flow.Child(CreateWorkingCopyFromRevision)
     request = flow.Child(Request)
+    compute_hash_action = flow.Child(ComputeRevisionHash)
+    check_hash = flow.Child(CheckRevisionHash)
 
     def get_path(self):
         return os.path.join(self.path.get(), self.file_name.get())
@@ -160,19 +219,23 @@ class Revision(flow.Object):
         if os.path.exists(path):
             with open(path, "rb") as f:
                 content = f.read()
-                self.hash.set(
-                    hashlib.md5(content).hexdigest()
-                )
-            self.hash.touch()
+
+            return hashlib.md5(content).hexdigest()
+    
+    def update_hash(self):
+        self.hash.set(self.compute_hash())
+        self.hash.touch()
+    
+    def hash_is_valid(self):
+        return self.hash.get() == self.compute_hash()
 
     def compute_child_value(self, child_value):
         if child_value is self.is_current:
             child_value.set(self.name() == self._file.current_revision.get())
         elif child_value is self.path:
-            file_path = get_contextual_dict(self._file, "settings")["path"]
             path = os.path.join(
                 self.root().project().get_root("UNKNOWN_ROOT_DIR"),
-                file_path,
+                self._file.path.get(),
                 self.name(),
             )
             child_value.set(path)
@@ -304,11 +367,14 @@ class Revisions(flow.Map):
             for s in item.sync.mapped_items():
                 site_name = s.get_short_name() if self._enable_short_names.get() else s.name()
                 style["%s_icon" % site_name] = color_and_icon_by_status[s.status.get()][1]
+        
+        style["Revision_activate_oid"] = item.open.oid()
 
 
 class History(flow.Object):
 
     revisions = flow.Child(Revisions).injectable().ui(expanded=True)
+    department = flow.Parent(3)
 
 
 class FileFormat(flow.values.ChoiceValue):
@@ -641,14 +707,15 @@ class OpenWithNotepadPPAction(OpenWithAction):
         return ["txt"]
 
 
-class PublishFileAction(flow.Action):
+class PublishFileAction(LaunchSessionWorker):
 
     ICON = ("icons.libreflow", "publish")
 
     _file = flow.Parent()
     _department = flow.Parent(3)
 
-    comment = flow.Param("")
+    comment = flow.SessionParam("")
+    upload_after_publish = flow.BoolParam(False)
 
     def get_buttons(self):
         self.message.set("<h3>Publish</h3>")
@@ -666,6 +733,26 @@ class PublishFileAction(flow.Action):
 
     def allow_context(self, context):
         return context and self._file.has_working_copy(True)
+    
+    def launcher_oid(self):
+        return self.oid()
+
+    def launcher_exec_func_name(self):
+        return "upload_published_revision"
+    
+    def upload_published_revision(self):
+        head = self._file.get_head_revision()
+        current_site = self.root().project().get_current_site()
+
+        upload_job = current_site.queue.submit_job(
+            emitter_oid=head.oid(),
+            user=self.root().project().get_user(),
+            studio=current_site.name(),
+            job_type='Upload',
+            init_status='WAITING'
+        )
+        process_jobs_action = self.root().project().admin.process_jobs
+        process_jobs_action._process(upload_job)
 
     def run(self, button):
         if button == "Cancel":
@@ -684,6 +771,9 @@ class PublishFileAction(flow.Action):
 
         self._file.touch()
 
+        if self.upload_after_publish.get():
+            super(PublishFileAction, self).run(None)
+
 
 class MakeFileCurrentAction(flow.Action):
 
@@ -799,15 +889,18 @@ class ActiveUsers(flow.Map):
 
 class FileSystemItem(flow.Object):
 
-    _parent = flow.Parent()
+    _department = flow.Parent(2)
     settings = flow.Child(ContextualView).ui(hidden=True)
+    path = flow.Computed(cached=True)
 
     def get_name(self):
         return self.name()
 
     def get_path(self):
-        settings = get_contextual_dict(self, "settings", ["path"])
-        return os.path.join(self.root().project().get_root(), settings["path"])
+        return os.path.join(
+            self.root().project().get_root(),
+            self.path.get()
+        )
 
     def get_last_edit_time(self):
         path = self.get_path()
@@ -815,11 +908,13 @@ class FileSystemItem(flow.Object):
             return os.path.getmtime(path)
         
         return 0
-
-    def get_default_contextual_edits(self, context_name):
-        if context_name == "settings":
-            settings = get_contextual_dict(self._parent, "settings", ["path"])
-            return dict(path=os.path.join(settings["path"], self.get_name()))
+    
+    def compute_child_value(self, child_value):
+        if child_value is self.path:
+            child_value.set(os.path.join(
+                self._department.path.get(),
+                self.get_name()
+            ))
 
 
 class File(FileSystemItem):
@@ -843,6 +938,8 @@ class File(FileSystemItem):
         if child_value is self.display_name:
             file_name = "_".join(self.name().split("_")[:-1])
             child_value.set("%s.%s" % (file_name, self.format.get()))
+        else:
+            super(File, self).compute_child_value(child_value)
 
 
 class SearchExistingRevisions(flow.Action):
@@ -876,7 +973,6 @@ class TrackedFile(File):
     _locked_by = flow.Param("")
 
     complete_name = flow.Param("").ui(editable=False)
-    path = flow.Computed()
     state = flow.Computed()
 
     history = flow.Child(History)
@@ -1137,7 +1233,7 @@ class TrackedFile(File):
         revisions.touch()
 
         # Compute published revision hash
-        head_revision.compute_hash()
+        head_revision.compute_hash_action.run(None)
 
         return head_revision
 
@@ -1215,11 +1311,7 @@ class TrackedFile(File):
     def compute_child_value(self, child_value):
         current_user = self.root().project().get_user()
 
-        if child_value is self.path:
-            settings = get_contextual_dict(self, "settings", ["path"])
-            self.path.set(os.path.join(self.root().project().get_root(), settings["path"]))
-
-        elif child_value is self.current_user_sees:
+        if child_value is self.current_user_sees:
             try:
                 child_value.set(self.active_users[current_user].get())
             except flow.exceptions.MappedNameError:
