@@ -1,4 +1,5 @@
 import os
+import sys
 import platform
 import subprocess
 from pathlib import Path
@@ -48,14 +49,20 @@ class BaseRunner(Runner):
         if not os.path.exists(self.get_log_path()):
             os.mkdir(self.get_log_path())
 
-        log_path = "%s/%s_%s.log" % (
+        log_out_path = "%s/%s_%s.log" % (
             self.get_log_path(),
             self.runner_name(),
             time.time(),
         )
-        with open(log_path, "w+") as log_file:
-            popen = subprocess.Popen(cmd, env=env, stdout=log_file, **os_flags)
-            self.popen = popen
+        log_err_path = "%s/%s_%s_error.log" % (
+            self.get_log_path(),
+            self.runner_name(),
+            time.time(),
+        )
+
+        with open(log_out_path, "w+") as out, open(log_err_path, "w+") as err:
+            popen = subprocess.Popen(cmd, env=env, stdout=out, stderr=err, **os_flags)
+            self._popen = popen
 
     def get_log_path(self):
         return "%s/.libreflow/log" % Path.home()
@@ -186,6 +193,47 @@ class Firefox(EditFileRunner):
         return True
 
 
+class PythonRunner(BaseRunner):
+
+    def executable(self):
+        return sys.executable
+    
+    def show_terminal(self):
+        return False
+
+    def keep_terminal(self):
+        return False
+
+
+class SessionWorker(PythonRunner):
+
+    def argv(self):
+        args = [
+            "C:/Users/Baptiste/dev/LibreFlow/libreflow-git/src/libreflow/scripts/session_worker.py",
+            self.runner_name()
+        ]
+        args += self.extra_argv
+        return args
+
+
+class LaunchSessionWorker(RunAction):
+
+    def runner_name_and_tags(self):
+        return "SessionWorker", []
+    
+    def launcher_oid(self):
+        raise NotImplementedError()
+
+    def launcher_exec_func_name(self):
+        raise NotImplementedError()
+    
+    def extra_argv(self):
+        return [
+            self.launcher_oid(),
+            self.launcher_exec_func_name()
+        ]
+
+
 class RunnerChoiceValue(flow.values.ChoiceValue):
 
     _extension = flow.Parent(2)
