@@ -12,7 +12,7 @@ from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
 from kabaret.subprocess_manager.flow import RunAction
 
 from .maputils import ItemMap, CreateGenericAction, ClearMapAction, RemoveGenericAction
-from libreflow.baseflow.runners import BaseRunner
+from libreflow.baseflow.runners import BaseRunner, PythonRunner
 
 
 class StaticSiteTypeChoices(flow.values.ChoiceValue):
@@ -188,17 +188,6 @@ class ProcessJobs(flow.Action):
             self._process(job)
 
 
-class PythonRunner(BaseRunner):
-
-    def executable(self):
-        return sys.executable
-    
-    def show_terminal(self):
-        return False
-
-    def keep_terminal(self):
-        return False
-
 class MinioFileUploader(PythonRunner):
     
     def argv(self):
@@ -362,16 +351,39 @@ class Synchronize(ProcessJobs):
         self.root().project().synchronization.touch()
 
 
+class SiteChoiceValue(flow.values.ChoiceValue):
+    _revision = flow.Parent(2)
+
+    def choices(self):
+        site_names = []
+        
+        for site in self._revision.sync.mapped_items():
+            if site.status.get() == "Available":
+                site_names.append(site.name())
+        
+        return site_names
+
+
 class Request(flow.Action):
 
     _revision = flow.Parent()
     priority = flow.IntParam(50)
+    site_name = flow.Param(None, SiteChoiceValue).ui(label="Site to query")
 
     def get_buttons(self):
+        site_names = self.site_name.choices()
+        # Actually, an existing revision implies there is at least one
+        # site on which it is available, which is the source site
+        self.site_name.set(site_names[0])
+
         return ["Request", "Cancel"]
     
     def allow_context(self, context):
-        return context and not self._revision.is_working_copy()
+        return (
+            context
+            and not self._revision.is_working_copy()
+            and not self._revision.get_sync_status() == "Available"
+        )
 
     def run(self, button):
         if button == "Cancel":
@@ -381,22 +393,9 @@ class Request(flow.Action):
         current_site = self.root().project().get_current_site()
         current_site_status = self._revision.sync[current_site.name()]
 
-        if current_site_status.status.get() != "NotAvailable":
-            self.message.set("Revision status: %s" % current_site_status.status.get())
-            return self.get_result(close=False)
-
         # Get requested site
         sites = self.root().project().admin.sites
-        requested_site = None
-
-        for site_status in self._revision.sync.mapped_items():
-            if site_status.status.get() == "Available":
-                self.root().session().log_debug("Found revision in site %s" % site_status.name())
-                requested_site = sites[site_status.name()]
-
-        if not requested_site:
-            self.message.set("Revision unavailable yet")
-            return self.get_result(close=False)
+        requested_site = sites[self.site_name.get()]
         
         # Add a download job for the requesting site
         current_site.queue.submit_job(
@@ -416,14 +415,20 @@ class Request(flow.Action):
             site_status = self._revision.sync[site_name]
 
             if site.site_type.get() == "Exchange" and site_status.status.get() == "Available":
-                self.message.set("Revision already available on exchange server")
-                return self.get_result(close=False)
+                self.root().session().log_warning(
+                    "Revision already available on exchange server"
+                )
+                self.root().project().synchronization.touch()
+                return self.get_result()
             
         # Check if the source version upload is not already requested
         for job in requested_site.queue.mapped_items():
             if job.emitter_oid.get() == self._revision.oid() and job.type.get() == "Upload":
-                self.message.set("Revision already requested for upload in source site")
-                return self.get_result(close=False)
+                self.root().session.log_warning(
+                    "Revision already requested for upload in source site"
+                )
+                self.root().project().synchronization.touch()
+                return self.get_result()
         
         # Add an upload job for the requested site
         requested_site.queue.submit_job(
@@ -499,7 +504,7 @@ class Site(flow.Object):
             else:
                 print("ERROR: Unrecognized operative system ?")
             
-            if not os.path.exists(root_dir):
+            if not root_dir or not os.path.exists(root_dir):
                 print("WARNING: ROOT_DIR path DOES NOT EXISTS")
 
             child_value.set(root_dir)
