@@ -198,23 +198,18 @@ class CreateTrackedFileAction(baseflow.file.CreateTrackedFileAction):
             return
 
         settings = get_contextual_dict(self, "settings")
+        file_category = settings.get("file_category", None)
 
         name = self.file_name.get()
         prefix = ""
 
-        for type in ["episode", "sequence", "shot"]:
-            if type in settings:
-                prefix += "%s_" % settings[type]
-        
-        if not prefix:
-            for type in ["asset_type", "asset_name"]:
-                if type in settings:
-                    prefix += "%s_" % settings[type]
-        
-        try:
-            prefix += "%s_" % settings["department"] 
-        except KeyError:
-            pass
+        if file_category is not None:
+            if file_category == "PROD":
+                prefix = "{episode}_{sequence}_{shot}_{department}_"
+            elif file_category == "LIB":
+                prefix = "{asset_name}_{department}_"
+
+            prefix = prefix.format(**settings)
 
         self.root().session().log_debug(
             "Creating file %s.%s" % (name, self.file_format.get())
